package com.link.core.converter;

import com.link.core.converter.util.Assert;
import com.link.core.converter.util.RegularCloneable;
import com.link.core.converter.util.StringUtil;
import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import javassist.ClassClassPath;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtConstructor;
import javassist.CtMethod;
import javassist.NotFoundException;

public class InjectorBuilder<S, T> {
   private static final int NO_CONVERTER = -1;
   private static final char NAME_SEPARATOR = '.';
   private static final String REGEX_NAME_SEPARATOR = "\\.";
   private static final String INDENT = "  ";
   private Class<? extends S> sourceType;
   private Class<? extends T> targetType;
   private List<InjectorBuilder<S, T>.Ruler> rulers = new ArrayList();
   private List<Converter<?, ?>> converters = new ArrayList();
   private List<Injector<?, ?>> extendedInjectors = new ArrayList();
   private List<InjectorFilter<? extends S, ? extends T>> filters = new ArrayList();

   public static <S, T> InjectorBuilder<S, T> newBuilder(Class<S> sourceType, Class<T> targetType) throws IllegalArgumentException {
      return new InjectorBuilder(sourceType, targetType);
   }

   static CtClass getCtClass(ClassPool cp, Class type) throws IllegalArgumentException, NotFoundException {
      if (type == null) {
         return null;
      } else {
         Assert.assertArgumentNotNull(cp, "cp");

         try {
            return cp.get(type.getName());
         } catch (NotFoundException var3) {
            cp.insertClassPath(new ClassClassPath(type));
            return cp.get(type.getName());
         }
      }
   }

   static CtClass getCtClass(ClassPool cp, String className) throws IllegalArgumentException {
      if (className == null) {
         return null;
      } else {
         Assert.assertArgumentNotNull(cp, "cp");

         try {
            return cp.get(className);
         } catch (NotFoundException var3) {
            return null;
         }
      }
   }

   InjectorBuilder(Class<? extends S> sourceType, Class<? extends T> targetType) throws IllegalArgumentException {
      Assert.assertArgumentNotNull(sourceType, "sourceType");
      Assert.assertArgumentNotNull(targetType, "targetType");
      this.sourceType = sourceType;
      this.targetType = targetType;
   }

   public Injector<S, T> build() throws ConversionException {
      Class<? extends Injector<S, T>> injectorType = this.buildType();

      try {
         Injector<S, T> injector = (Injector)injectorType.newInstance();
         this.initialInjector(injector);
         return injector;
      } catch (ConversionException var3) {
         ConversionException e = var3;
         throw e;
      } catch (Exception var4) {
         Exception e = var4;
         throw new ConversionException(e);
      }
   }

   Class<? extends Injector<S, T>> buildType() throws ConversionException {
      try {
         ClassPool cp = ClassPool.getDefault();
         String className = this.generateClassName();
         CtClass implClass = getCtClass(cp, className);
         if (implClass != null) {
            return Class.forName(className);
         } else {
            implClass = cp.makeClass(className);
            CtClass assistClass = getCtClass(cp, InjectorAssist.class);
            implClass.setSuperclass(assistClass);
            CtClass injectorIntf = getCtClass(cp, Injector.class);
            implClass.addInterface(injectorIntf);
            CtConstructor constructor = new CtConstructor(new CtClass[0], implClass);
            constructor.setModifiers(1);
            constructor.setBody("{}");
            implClass.addConstructor(constructor);
            CtClass objectClass = getCtClass(cp, Object.class);
            CtMethod injectMethod = new CtMethod(CtClass.voidType, "inject", new CtClass[]{objectClass, objectClass}, implClass);
            injectMethod.setModifiers(1);
            injectMethod.setBody(this.buildInjectBody());
            implClass.addMethod(injectMethod);
            CtMethod cloneMethod = new CtMethod(objectClass, "clone", new CtClass[0], implClass);
            cloneMethod.setModifiers(1);
            cloneMethod.setBody(this.buildCloneBody(className));
            implClass.addMethod(cloneMethod);
            return implClass.toClass();
         }
      } catch (ConversionException var10) {
         ConversionException e = var10;
         throw e;
      } catch (Exception var11) {
         Exception e = var11;
         throw new ConversionException(e);
      }
   }

   void initialInjector(Injector<S, T> injector) throws IllegalArgumentException, ClassCastException {
      Assert.assertArgumentNotNull(injector, "injector");
      ((InjectorAssist)injector).setBuilder(this);
      ((InjectorAssist)injector).getConverters().addAll(this.converters);
      ((InjectorAssist)injector).getExtendedInjectors().addAll(this.extendedInjectors);
      ((InjectorAssist)injector).getFilters().addAll(this.filters);
   }

   public InjectorBuilder<S, T> map(String sourcePropertyName, String targetPropertyName, Converter<?, ?> converter) {
      InjectorBuilder<S, T>.NameRuler ruler = new NameRuler();
      ruler.setSourceName(sourcePropertyName);
      ruler.setTargetName(targetPropertyName);
      if (converter != null && !this.converters.contains(converter)) {
         this.converters.add(converter);
      }

      ruler.setConverter(this.converters.indexOf(converter));
      this.rulers.add(ruler);
      return this;
   }

   public InjectorBuilder<S, T> map(String sourcePropertyName, String targetPropertyName) {
      return this.map((String)sourcePropertyName, (String)targetPropertyName, (Converter)null);
   }

   public InjectorBuilder<S, T> map(String propertyName, Converter<?, ?> converter) {
      return this.map(propertyName, propertyName, converter);
   }

   public <SP, TP> InjectorBuilder<S, T> map(Class<SP> sourcePropertyType, Class<TP> targetPropertyType, Converter<?, ?> converter) throws IllegalArgumentException {
      return this.map(sourcePropertyType, false, targetPropertyType, false, converter);
   }

   public <SP, TP> InjectorBuilder<S, T> map(Class<SP> sourcePropertyType, Class<TP> targetPropertyType, Converter<?, ?> converter, boolean polymorphic) throws IllegalArgumentException {
      return this.map(sourcePropertyType, polymorphic, targetPropertyType, polymorphic, converter);
   }

   public <SP, TP> InjectorBuilder<S, T> map(Class<SP> sourcePropertyType, boolean sourcePolymorphic, Class<TP> targetPropertyType, boolean targetPolymorphic, Converter<?, ?> converter) throws IllegalArgumentException {
      Assert.assertArgumentNotNull(sourcePropertyType, "sourcePropertyType");
      Assert.assertArgumentNotNull(targetPropertyType, "targetPropertyType");
      InjectorBuilder<S, T>.TypeRuler ruler = new TypeRuler();
      ruler.setSourceType(sourcePropertyType);
      ruler.setSourcePolymorphic(sourcePolymorphic);
      ruler.setTargetType(targetPropertyType);
      ruler.setTargetPolymorphic(targetPolymorphic);
      if (converter != null && !this.converters.contains(converter)) {
         this.converters.add(converter);
      }

      ruler.setConverter(this.converters.indexOf(converter));
      this.rulers.add(ruler);
      return this;
   }

   public InjectorBuilder<S, T> ignore(String... targetPropertyNames) {
      String[] var2 = targetPropertyNames;
      int var3 = targetPropertyNames.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         String targetPropertyName = var2[var4];
         this.map((String)null, (String)targetPropertyName);
      }

      return this;
   }

   public InjectorBuilder<S, T> inclouds(String... targetPropertyNames) {
      Set<String> names = (Set)Arrays.stream(targetPropertyNames).collect(Collectors.toSet());
      this.filters.add((filter) -> {
         filter.setCancelled(!names.contains(filter.getTargetPropertyName()));
      });
      return this;
   }

   public InjectorBuilder<S, T> extend(Injector<?, ?> injector) throws IllegalArgumentException {
      Assert.assertArgumentNotNull(injector, "injector");
      this.extendedInjectors.add(injector);
      return this;
   }

   public InjectorBuilder<S, T> addFilter(InjectorFilter<? extends S, ? extends T> filter) throws IllegalArgumentException {
      Assert.assertArgumentNotNull(filter, "filter");
      this.filters.add(filter);
      return this;
   }

   public Class<? extends S> getSourceType() {
      return this.sourceType;
   }

   public Class<? extends T> getTargetType() {
      return this.targetType;
   }

   int getTypeFeature() {
      return this.rulers.hashCode();
   }

   public InjectorBuilder<S, T> clone() {
      InjectorBuilder<S, T> other = new InjectorBuilder(this.sourceType, this.targetType);
      other.rulers.clear();
      other.rulers.addAll(this.rulers);
      other.converters.clear();
      other.converters.addAll(this.converters);
      other.extendedInjectors.clear();
      other.extendedInjectors.addAll(this.extendedInjectors);
      other.filters.clear();
      other.filters.addAll(this.filters);
      return other;
   }

   private String generateClassName() {
      StringBuffer sb = new StringBuffer();
      sb.append(this.targetType.getName());
      sb.append("Injector$");
      sb.append(this.sourceType.getName().replace('.', '_'));
      sb.append('$');
      String feature = Integer.toHexString(this.getTypeFeature());
      sb.append(feature);
      return sb.toString();
   }

   private String buildInjectBody() throws IntrospectionException {
      StringBuffer sb = new StringBuffer();
      sb.append("{\n");
      sb.append("  if ($1 == null) {\n");
      sb.append("    throw new IllegalArgumentException(\"The parameter 'source' cannot be null.\");\n");
      sb.append("  }\n");
      sb.append("  if ($2 == null) {\n");
      sb.append("    throw new IllegalArgumentException(\"The parameter 'target' cannot be null.\");\n");
      sb.append("  }\n");
      sb.append("  ");
      sb.append(this.sourceType.getName());
      sb.append(" source = (");
      sb.append(this.sourceType.getName());
      sb.append(") $1;\n");
      sb.append("  ");
      sb.append(this.targetType.getName());
      sb.append(" target = (");
      sb.append(this.targetType.getName());
      sb.append(") $2;\n");
      sb.append(this.buildAssignments("source", "target"));
      sb.append("  ");
      sb.append("injectExtends_($1, $2);\n");
      sb.append("}");
      return sb.toString();
   }

   private String buildAssignments(String sourceVar, String targetVar) throws IntrospectionException {
      InjectorBuilder<S, T>.Assignments assignments = new Assignments();
      assignments.initial();
      Iterator var4 = this.rulers.iterator();

      while(var4.hasNext()) {
         InjectorBuilder<S, T>.Ruler ruler = (Ruler)var4.next();
         ruler.apply(assignments);
      }

      String java = assignments.toJava(sourceVar, targetVar);
      return this.indent(java);
   }

   private String buildCloneBody(String className) {
      assert !StringUtil.isNullOrBlank(className);

      StringBuffer sb = new StringBuffer();
      sb.append("{\n");
      sb.append("  ");
      sb.append(className);
      sb.append(" other = new ");
      sb.append(className);
      sb.append("();\n");
      sb.append("  other.copyFrom(this);\n");
      sb.append("  return other;\n");
      sb.append("}");
      return sb.toString();
   }

   private String indent(String java) {
      assert java != null;

      String REGEX_LINE_BREAK = "\\n";
      String[] lines = java.split("\\n");
      StringBuffer sb = new StringBuffer();

      for(int index = 0; index < lines.length; ++index) {
         String line = lines[index];
         if (!"".equals(line) || index != lines.length - 1) {
            sb.append("  ");
            sb.append(lines[index]);
            sb.append("\n");
         }
      }

      return sb.toString();
   }

   private class Assignments extends ArrayList<InjectorBuilder<S, T>.Assignment> {
      private static final long serialVersionUID = 8026887767417957350L;

      private Assignments() {
      }

      public void initial() throws IntrospectionException {
         BeanInfo sbi = Introspector.getBeanInfo(InjectorBuilder.this.sourceType);
         BeanInfo tbi = Introspector.getBeanInfo(InjectorBuilder.this.targetType);
         PropertyDescriptor[] var3 = tbi.getPropertyDescriptors();
         int var4 = var3.length;

         for(int var5 = 0; var5 < var4; ++var5) {
            PropertyDescriptor tpd = var3[var5];
            if (tpd.getWriteMethod() != null) {
               String name = tpd.getName();
               PropertyDescriptor matchedSpd = null;
               PropertyDescriptor[] var9 = sbi.getPropertyDescriptors();
               int var10 = var9.length;

               for(int var11 = 0; var11 < var10; ++var11) {
                  PropertyDescriptor spd = var9[var11];
                  if (name.equals(spd.getName())) {
                     matchedSpd = spd;
                     break;
                  }
               }

               if (matchedSpd != null && matchedSpd.getReadMethod() != null) {
                  InjectorBuilder<S, T>.Assignment assignment = InjectorBuilder.this.new Assignment();
                  assignment.getTpds().add(tpd);
                  assignment.getSpds().add(matchedSpd);
                  assignment.setConverter(-1);
                  this.add(assignment);
               }
            }
         }

      }

      public InjectorBuilder<S, T>.Assignment getByTargetName(String targetName) {
         Iterator var2 = this.iterator();

         Assignment assignment;
         do {
            if (!var2.hasNext()) {
               return null;
            }

            assignment = (Assignment)var2.next();
         } while(!this.objectEquals(targetName, assignment.getTargetName()));

         return assignment;
      }

      public boolean objectEquals(Object object1, Object object2) {
         if (object1 == object2) {
            return true;
         } else {
            return object1 != null && object2 != null ? object1.equals(object2) : false;
         }
      }

      public InjectorBuilder<S, T>.Assignment ensure(String targetName) throws ConversionException, IntrospectionException {
         InjectorBuilder<S, T>.Assignment assignment = this.getByTargetName(targetName);
         if (assignment != null) {
            return assignment;
         } else {
            assignment = this.createByTargetName(targetName);
            this.removeSubtree(assignment.getTargetName());
            this.add(assignment);
            return assignment;
         }
      }

      public String toJava(String sourceVar, String targetVar) {
         StringBuffer sb = new StringBuffer();
         Iterator var4 = this.iterator();

         while(var4.hasNext()) {
            InjectorBuilder<S, T>.Assignment assignment = (Assignment)var4.next();
            sb.append(assignment.toJava(sourceVar, targetVar));
         }

         return sb.toString();
      }

      private InjectorBuilder<S, T>.Assignment createByTargetName(String targetName) throws ConversionException, IntrospectionException {
         if (targetName == null) {
            throw new ConversionException(MessageFormat.format("在目标对象“{0}”找不到属性“{1}”。", InjectorBuilder.this.targetType.getName(), targetName), new Object[0]);
         } else {
            InjectorBuilder<S, T>.Assignment assignment = InjectorBuilder.this.new Assignment();
            String[] words = targetName.split("\\.");
            PropertyDescriptor lastPd = null;
            String[] var5 = words;
            int var6 = words.length;

            for(int var7 = 0; var7 < var6; ++var7) {
               String word = var5[var7];
               word = word.trim();
               if (!"".equals(word)) {
                  BeanInfo bi = null;
                  if (lastPd == null) {
                     bi = Introspector.getBeanInfo(InjectorBuilder.this.targetType);
                  } else {
                     bi = Introspector.getBeanInfo(lastPd.getPropertyType());
                  }

                  PropertyDescriptor foundPd = null;
                  PropertyDescriptor[] var11 = bi.getPropertyDescriptors();
                  int var12 = var11.length;

                  for(int var13 = 0; var13 < var12; ++var13) {
                     PropertyDescriptor pd = var11[var13];
                     if (word.equals(pd.getName())) {
                        foundPd = pd;
                        break;
                     }
                  }

                  if (foundPd == null) {
                     throw new ConversionException(MessageFormat.format("在目标对象“{0}”找不到属性“{1}”。", InjectorBuilder.this.targetType.getName(), targetName), new Object[0]);
                  }

                  assignment.getTpds().add(foundPd);
                  lastPd = foundPd;
               }
            }

            if (assignment.getTpds().isEmpty()) {
               throw new ConversionException(MessageFormat.format("在目标对象“{0}”找不到属性“{1}”。", InjectorBuilder.this.targetType.getName(), targetName), new Object[0]);
            } else {
               return assignment;
            }
         }
      }

      private void removeSubtree(String targetName) {
         assert targetName != null;

         String prefix = targetName + ".";

         for(int index = this.size() - 1; index >= 0; --index) {
            InjectorBuilder<S, T>.Assignment assignment = (Assignment)this.get(index);
            if (assignment.getTargetName().startsWith(prefix)) {
               this.remove(index);
            }
         }

      }

      // $FF: synthetic method
      Assignments(Object x1) {
         this();
      }
   }

   private class Assignment {
      private List<PropertyDescriptor> spds;
      private List<PropertyDescriptor> tpds;
      private int converter;

      private Assignment() {
         this.spds = new ArrayList();
         this.tpds = new ArrayList();
         this.converter = -1;
      }

      public List<PropertyDescriptor> getSpds() {
         return this.spds;
      }

      private PropertyDescriptor getSpd() {
         return this.spds.isEmpty() ? null : (PropertyDescriptor)this.spds.get(this.spds.size() - 1);
      }

      private String getSourceName() {
         StringBuffer sb = new StringBuffer();

         PropertyDescriptor pd;
         for(Iterator var2 = this.spds.iterator(); var2.hasNext(); sb.append(pd.getName())) {
            pd = (PropertyDescriptor)var2.next();
            if (sb.length() > 0) {
               sb.append('.');
            }
         }

         return sb.toString();
      }

      public void setSourceName(String sourceName) throws ConversionException, IntrospectionException {
         this.spds.clear();
         if (!StringUtil.isNullOrEmpty(sourceName)) {
            String[] words = sourceName.split("\\.");
            PropertyDescriptor lastPd = null;
            String[] var4 = words;
            int var5 = words.length;

            for(int var6 = 0; var6 < var5; ++var6) {
               String word = var4[var6];
               if (!"".equals(word.trim())) {
                  PropertyDescriptor foundPd = null;
                  BeanInfo bi = null;
                  if (lastPd == null) {
                     bi = Introspector.getBeanInfo(InjectorBuilder.this.sourceType);
                  } else {
                     bi = Introspector.getBeanInfo(lastPd.getPropertyType());
                  }

                  PropertyDescriptor[] var10 = bi.getPropertyDescriptors();
                  int var11 = var10.length;

                  for(int var12 = 0; var12 < var11; ++var12) {
                     PropertyDescriptor pd = var10[var12];
                     if (word.equals(pd.getName())) {
                        foundPd = pd;
                        break;
                     }
                  }

                  if (foundPd == null) {
                     throw new ConversionException(MessageFormat.format("在源对象“{0}”中找不到属性“{1}”。", InjectorBuilder.this.sourceType.getName(), sourceName), new Object[0]);
                  }

                  this.spds.add(foundPd);
                  lastPd = foundPd;
               }
            }

         }
      }

      public List<PropertyDescriptor> getTpds() {
         return this.tpds;
      }

      private PropertyDescriptor getTpd() {
         return this.tpds.isEmpty() ? null : (PropertyDescriptor)this.tpds.get(this.tpds.size() - 1);
      }

      public String getTargetName() {
         StringBuffer sb = new StringBuffer();

         PropertyDescriptor pd;
         for(Iterator var2 = this.tpds.iterator(); var2.hasNext(); sb.append(pd.getName())) {
            pd = (PropertyDescriptor)var2.next();
            if (sb.length() > 0) {
               sb.append('.');
            }
         }

         return sb.toString();
      }

      public void setConverter(int converter) {
         this.converter = converter;
      }

      public String toString() {
         StringBuffer sb = new StringBuffer();
         sb.append(this.getSourceName());
         sb.append(" => ");
         sb.append(this.getTargetName());
         sb.append(" ");
         sb.append(this.converter);
         return sb.toString();
      }

      public String toJava(String sourceVar, String targetVar) {
         if (this.spds.isEmpty()) {
            return "";
         } else {
            String java = this.encodeSourceGet(sourceVar);
            if (this.converter != -1) {
               if (this.getSpd().getPropertyType().isPrimitive()) {
                  java = this.decorateBox(java);
               }

               java = this.decorateConverter(java);
               if (this.getTpd().getPropertyType().isPrimitive()) {
                  java = this.decorateUnbox(java);
               }
            } else if (this.getSpd().getPropertyType().isPrimitive() && !this.getTpd().getPropertyType().isPrimitive()) {
               java = this.decorateBox(java);
            } else if (!this.getSpd().getPropertyType().isPrimitive() && this.getTpd().getPropertyType().isPrimitive()) {
               java = this.decorateUnbox(java);
            }

            java = this.decorateTargetSet(targetVar, java);
            java = this.decorateTargetNullSafe(targetVar, java);
            java = this.decorateSourceNullSafe(sourceVar, java);
            if (!InjectorBuilder.this.filters.isEmpty()) {
               java = this.decoratePreprocess(java);
            }

            return java;
         }
      }

      private String encodeSourceGet(String sourceVar) {
         assert sourceVar != null;

         StringBuffer sb = new StringBuffer();
         sb.append(sourceVar);
         Iterator var3 = this.spds.iterator();

         while(var3.hasNext()) {
            PropertyDescriptor spd = (PropertyDescriptor)var3.next();
            sb.append('.');
            sb.append(spd.getReadMethod().getName());
            sb.append("()");
         }

         return sb.toString();
      }

      private String decorateConverter(String innerJava) {
         assert innerJava != null;

         StringBuffer sb = new StringBuffer();
         sb.append('(');
         sb.append(this.getConvertReturnType().getName());
         sb.append(") ");
         sb.append("convertProperty_(");
         sb.append(this.converter);
         sb.append(", ");
         sb.append(innerJava);
         sb.append(")");
         return sb.toString();
      }

      private Class getConvertReturnType() {
         Class type = this.getTpd().getPropertyType();
         if (Boolean.TYPE.equals(type)) {
            return Boolean.class;
         } else if (Byte.TYPE.equals(type)) {
            return Byte.class;
         } else if (Character.TYPE.equals(type)) {
            return Character.class;
         } else if (Double.TYPE.equals(type)) {
            return Double.class;
         } else if (Float.TYPE.equals(type)) {
            return Float.class;
         } else if (Integer.TYPE.equals(type)) {
            return Integer.class;
         } else if (Long.TYPE.equals(type)) {
            return Long.class;
         } else {
            return Short.TYPE.equals(type) ? Short.class : type;
         }
      }

      private String decorateBox(String innerJava) {
         StringBuffer sb = new StringBuffer();
         Class type = this.getSpd().getPropertyType();
         if (Boolean.TYPE.equals(type)) {
            sb.append("Boolean");
         } else if (Byte.TYPE.equals(type)) {
            sb.append("Byte");
         } else if (Character.TYPE.equals(type)) {
            sb.append("Character");
         } else if (Double.TYPE.equals(type)) {
            sb.append("Double");
         } else if (Float.TYPE.equals(type)) {
            sb.append("Float");
         } else if (Integer.TYPE.equals(type)) {
            sb.append("Integer");
         } else if (Long.TYPE.equals(type)) {
            sb.append("Long");
         } else if (Short.TYPE.equals(type)) {
            sb.append("Short");
         }

         sb.append(".valueOf(");
         sb.append(innerJava);
         sb.append(")");
         return sb.toString();
      }

      private String decorateUnbox(String innerJava) {
         StringBuffer sb = new StringBuffer();
         sb.append("(");
         sb.append(innerJava);
         sb.append(")");
         Class type = this.getTpd().getPropertyType();
         if (Boolean.TYPE.equals(type)) {
            sb.append(".booleanValue()");
         } else if (Byte.TYPE.equals(type)) {
            sb.append(".byteValue()");
         } else if (Character.TYPE.equals(type)) {
            sb.append(".charValue()");
         } else if (Double.TYPE.equals(type)) {
            sb.append(".doubleValue()");
         } else if (Float.TYPE.equals(type)) {
            sb.append(".floatValue()");
         } else if (Integer.TYPE.equals(type)) {
            sb.append(".intValue()");
         } else if (Long.TYPE.equals(type)) {
            sb.append(".longValue()");
         } else if (Short.TYPE.equals(type)) {
            sb.append(".shortValue()");
         }

         return sb.toString();
      }

      private String decorateTargetSet(String targetVar, String innerJava) {
         assert targetVar != null;

         assert innerJava != null;

         StringBuffer sb = new StringBuffer();
         sb.append(targetVar);

         assert this.tpds.size() > 0;

         int index;
         PropertyDescriptor lastTpd;
         for(index = 0; index < this.tpds.size() - 1; ++index) {
            sb.append(".");
            lastTpd = (PropertyDescriptor)this.tpds.get(index);
            sb.append(lastTpd.getReadMethod().getName());
            sb.append("()");
         }

         lastTpd = (PropertyDescriptor)this.tpds.get(index);
         sb.append(".");
         sb.append(lastTpd.getWriteMethod().getName());
         sb.append("(");
         sb.append(innerJava);
         sb.append(");\n");
         return sb.toString();
      }

      private String decorateSourceNullSafe(String sourceVar, String innerJava) {
         assert innerJava != null;

         if (this.spds.size() <= 1) {
            return innerJava;
         } else {
            StringBuffer sb = new StringBuffer();
            sb.append("if (");

            for(int endIndex = 0; endIndex < this.spds.size() - 1; ++endIndex) {
               if (endIndex > 0) {
                  sb.append(" && ");
               }

               sb.append(sourceVar);

               for(int index = 0; index <= endIndex; ++index) {
                  sb.append(".");
                  PropertyDescriptor spd = (PropertyDescriptor)this.spds.get(index);
                  sb.append(spd.getReadMethod().getName());
                  sb.append("()");
               }

               sb.append(" != null");
            }

            sb.append(") {\n");
            sb.append(InjectorBuilder.this.indent(innerJava));
            sb.append("};\n");
            return sb.toString();
         }
      }

      private String decorateTargetNullSafe(String targetVar, String innerJava) {
         assert targetVar != null;

         assert innerJava != null;

         if (this.tpds.size() <= 1) {
            return innerJava;
         } else {
            StringBuffer sb = new StringBuffer();

            for(int endIndex = 0; endIndex < this.tpds.size() - 1; ++endIndex) {
               sb.append("if (");
               sb.append(targetVar);

               int index;
               PropertyDescriptor tpd;
               for(index = 0; index <= endIndex; ++index) {
                  sb.append(".");
                  tpd = (PropertyDescriptor)this.tpds.get(index);
                  sb.append(tpd.getReadMethod().getName());
                  sb.append("()");
               }

               sb.append(" == null) {\n");
               sb.append("  ");
               sb.append(targetVar);

               for(index = 0; index <= endIndex - 1; ++index) {
                  sb.append(".");
                  tpd = (PropertyDescriptor)this.tpds.get(index);
                  sb.append(tpd.getReadMethod().getName());
                  sb.append("()");
               }

               sb.append(".");
               PropertyDescriptor tpdx = (PropertyDescriptor)this.tpds.get(endIndex);
               sb.append(tpdx.getWriteMethod().getName());
               sb.append("(new ");
               sb.append(tpdx.getPropertyType().getName());
               sb.append("());\n");
               sb.append("}\n");
            }

            sb.append(innerJava);
            return sb.toString();
         }
      }

      private String decoratePreprocess(String innerJava) {
         assert innerJava != null;

         StringBuffer sb = new StringBuffer();
         sb.append("if (preprocess_($1, \"");
         sb.append(this.getSourceName());
         sb.append("\", $2, \"");
         sb.append(this.getTargetName());
         sb.append("\")) {\n");
         sb.append(InjectorBuilder.this.indent(innerJava));
         sb.append("}\n");
         return sb.toString();
      }

      // $FF: synthetic method
      Assignment(Object x1) {
         this();
      }
   }

   private class TypeRuler extends InjectorBuilder<S, T>.Ruler {
      private Class<?> sourceType;
      private boolean sourcePolymorphic;
      private Class<?> targetType;
      private boolean targetPolymorphic;

      private TypeRuler() {
         super(null);
         this.sourcePolymorphic = false;
         this.targetPolymorphic = false;
      }

      public void setSourceType(Class<?> sourceType) {
         this.sourceType = sourceType;
      }

      public void setSourcePolymorphic(boolean sourcePolymorphic) {
         this.sourcePolymorphic = sourcePolymorphic;
      }

      public void setTargetType(Class<?> targetType) {
         this.targetType = targetType;
      }

      public void setTargetPolymorphic(boolean targetPolymorphic) {
         this.targetPolymorphic = targetPolymorphic;
      }

      public void apply(InjectorBuilder<S, T>.Assignments assignments) {
         assert assignments != null;

         Iterator var2 = assignments.iterator();

         while(var2.hasNext()) {
            InjectorBuilder<S, T>.Assignment assignment = (Assignment)var2.next();
            if (assignment.getSpds().size() == 1 && assignment.getTpds().size() == 1) {
               PropertyDescriptor spd = (PropertyDescriptor)assignment.getSpds().get(0);
               PropertyDescriptor tpd = (PropertyDescriptor)assignment.getTpds().get(0);

               assert spd != null;

               assert tpd != null;

               if (this.matchedPropertyType(this.sourceType, spd.getPropertyType(), this.sourcePolymorphic) && this.matchedPropertyType(this.targetType, tpd.getPropertyType(), this.targetPolymorphic)) {
                  assignment.setConverter(this.getConverter());
               }
            }
         }

      }

      private boolean matchedPropertyType(Class expected, Class actual, boolean polymorphic) {
         assert expected != null;

         assert actual != null;

         return polymorphic ? expected.isAssignableFrom(actual) : expected.equals(actual);
      }

      public int hashCode() {
         int prime = true;
         int result = super.hashCode();
         result = 31 * result + this.getOuterType().hashCode();
         result = 31 * result + (this.sourceType == null ? 0 : this.sourceType.getName().hashCode());
         result = 31 * result + (this.sourcePolymorphic ? 1231 : 1237);
         result = 31 * result + (this.targetType == null ? 0 : this.targetType.getName().hashCode());
         result = 31 * result + (this.targetPolymorphic ? 1231 : 1237);
         return result;
      }

      public boolean equals(Object obj) {
         if (this == obj) {
            return true;
         } else if (!super.equals(obj)) {
            return false;
         } else if (this.getClass() != obj.getClass()) {
            return false;
         } else {
            InjectorBuilder<S, T>.TypeRuler other = (TypeRuler)obj;
            if (!this.getOuterType().equals(other.getOuterType())) {
               return false;
            } else if (this.sourceType != other.sourceType) {
               return false;
            } else if (this.sourcePolymorphic != other.sourcePolymorphic) {
               return false;
            } else if (this.targetType != this.sourceType) {
               return false;
            } else {
               return this.targetPolymorphic == other.targetPolymorphic;
            }
         }
      }

      public String toString() {
         StringBuffer sb = new StringBuffer();
         sb.append(this.sourceType);
         sb.append(" => ");
         sb.append(this.targetType);
         sb.append(" ");
         sb.append(this.getConverter());
         return sb.toString();
      }

      private InjectorBuilder getOuterType() {
         return InjectorBuilder.this;
      }

      // $FF: synthetic method
      TypeRuler(Object x1) {
         this();
      }
   }

   private class NameRuler extends InjectorBuilder<S, T>.Ruler {
      private String sourceName;
      private String targetName;

      private NameRuler() {
         super(null);
      }

      public void setSourceName(String sourceName) {
         this.sourceName = this.normalize(sourceName);
      }

      public void setTargetName(String targetName) {
         this.targetName = this.normalize(targetName);
      }

      public void apply(InjectorBuilder<S, T>.Assignments assignments) throws ConversionException, IntrospectionException {
         assert assignments != null;

         InjectorBuilder<S, T>.Assignment assignment = assignments.ensure(this.targetName);
         assignment.setSourceName(this.sourceName);
         assignment.setConverter(this.getConverter());
      }

      public String toString() {
         StringBuffer sb = new StringBuffer();
         sb.append(this.sourceName);
         sb.append(" => ");
         sb.append(this.targetName);
         sb.append(" ");
         sb.append(this.getConverter());
         return sb.toString();
      }

      private String normalize(String name) {
         if (name == null) {
            return "";
         } else {
            String[] words = name.split("\\.");
            StringBuffer sb = new StringBuffer();
            String[] var4 = words;
            int var5 = words.length;

            for(int var6 = 0; var6 < var5; ++var6) {
               String word = var4[var6];
               word = word.trim();
               if (!"".equals(word)) {
                  if (sb.length() > 0) {
                     sb.append('.');
                  }

                  sb.append(word);
               }
            }

            return sb.toString();
         }
      }

      public int hashCode() {
         int prime = true;
         int result = super.hashCode();
         result = 31 * result + this.getOuterType().hashCode();
         result = 31 * result + (this.sourceName == null ? 0 : this.sourceName.hashCode());
         result = 31 * result + (this.targetName == null ? 0 : this.targetName.hashCode());
         return result;
      }

      public boolean equals(Object obj) {
         if (this == obj) {
            return true;
         } else if (!super.equals(obj)) {
            return false;
         } else if (this.getClass() != obj.getClass()) {
            return false;
         } else {
            InjectorBuilder<S, T>.NameRuler other = (NameRuler)obj;
            if (!this.getOuterType().equals(other.getOuterType())) {
               return false;
            } else {
               if (this.sourceName == null) {
                  if (other.sourceName != null) {
                     return false;
                  }
               } else if (!this.sourceName.equals(other.sourceName)) {
                  return false;
               }

               if (this.targetName == null) {
                  if (other.targetName != null) {
                     return false;
                  }
               } else if (!this.targetName.equals(other.targetName)) {
                  return false;
               }

               return true;
            }
         }
      }

      private InjectorBuilder getOuterType() {
         return InjectorBuilder.this;
      }

      // $FF: synthetic method
      NameRuler(Object x1) {
         this();
      }
   }

   private abstract class Ruler {
      private int converter;

      private Ruler() {
         this.converter = -1;
      }

      public int getConverter() {
         return this.converter;
      }

      public void setConverter(int converter) {
         this.converter = converter;
      }

      public abstract void apply(InjectorBuilder<S, T>.Assignments assignments) throws ConversionException, IntrospectionException;

      public int hashCode() {
         int prime = true;
         int result = 1;
         result = 31 * result + this.getOuterType().hashCode();
         result = 31 * result + this.converter;
         return result;
      }

      public boolean equals(Object obj) {
         if (this == obj) {
            return true;
         } else if (obj == null) {
            return false;
         } else if (this.getClass() != obj.getClass()) {
            return false;
         } else {
            InjectorBuilder<S, T>.Ruler other = (Ruler)obj;
            if (!this.getOuterType().equals(other.getOuterType())) {
               return false;
            } else {
               return this.converter == other.converter;
            }
         }
      }

      private InjectorBuilder getOuterType() {
         return InjectorBuilder.this;
      }

      // $FF: synthetic method
      Ruler(Object x1) {
         this();
      }
   }

   public abstract static class InjectorAssist implements HasConverters, RegularCloneable, HasInjectorBuilder {
      private InjectorBuilder<?, ?> builder;
      private List<Converter> converters = new ArrayList();
      private List<Injector<?, ?>> extendInjectors = new ArrayList();
      private List<InjectorFilter<?, ?>> filters = new ArrayList();

      public InjectorBuilder<?, ?> getBuilder() {
         return this.builder;
      }

      public void setBuilder(InjectorBuilder<?, ?> builder) {
         this.builder = builder;
      }

      public List<Converter> getConverters() {
         return this.converters;
      }

      protected Object convertProperty_(int converterIndex, Object sourcePropertyValue) {
         return ((Converter)this.converters.get(converterIndex)).convert(sourcePropertyValue);
      }

      public List<Injector<?, ?>> getExtendedInjectors() {
         return this.extendInjectors;
      }

      protected void injectExtends_(Object source, Object target) {
         Iterator var3 = this.extendInjectors.iterator();

         while(var3.hasNext()) {
            Injector extendInjector = (Injector)var3.next();
            extendInjector.inject(source, target);
         }

      }

      public List<InjectorFilter<?, ?>> getFilters() {
         return this.filters;
      }

      protected boolean preprocess(Object source, String sourcePropertyName, Object target, String targetPropertyName) {
         if (this.filters.isEmpty()) {
            return true;
         } else {
            InjectorFilterContext context = new InjectorFilterContext();
            context.setSource(source);
            context.setSourcePropertyName(sourcePropertyName);
            context.setTarget(target);
            context.setTargetPropertyName(targetPropertyName);
            Iterator var6 = this.filters.iterator();

            do {
               if (!var6.hasNext()) {
                  return true;
               }

               InjectorFilter<?, ?> filter = (InjectorFilter)var6.next();
               filter.filter(context);
            } while(!context.isCancelled());

            return false;
         }
      }

      public abstract Object clone();

      public void copyFrom(Object source) {
         if (source != null && source instanceof InjectorAssist) {
            InjectorAssist injector = (InjectorAssist)source;
            this.builder = injector.builder;
            this.converters.clear();

            Iterator var3;
            Converter tf;
            for(var3 = injector.converters.iterator(); var3.hasNext(); this.converters.add(tf)) {
               Converter scvt = (Converter)var3.next();
               tf = null;
               if (scvt instanceof RegularCloneable) {
                  tf = (Converter)((RegularCloneable)scvt).clone();
               } else {
                  tf = scvt;
               }
            }

            this.extendInjectors.clear();

            Injector tei;
            for(var3 = injector.extendInjectors.iterator(); var3.hasNext(); this.extendInjectors.add(tei)) {
               Injector sei = (Injector)var3.next();
               tf = null;
               if (sei instanceof RegularCloneable) {
                  tei = (Injector)((RegularCloneable)sei).clone();
               } else {
                  tei = sei;
               }
            }

            this.filters.clear();

            InjectorFilter tf;
            for(var3 = injector.filters.iterator(); var3.hasNext(); this.filters.add(tf)) {
               InjectorFilter sf = (InjectorFilter)var3.next();
               tf = null;
               if (sf instanceof RegularCloneable) {
                  tf = (InjectorFilter)((RegularCloneable)sf).clone();
               } else {
                  tf = sf;
               }
            }

         }
      }
   }
}
